package exceptions;

public class Datum implements Date {
	
	private int day; 
    private int month;   
    private int year;
    
    private static final int[] DAYS = { 0, 31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };
	
    /* check the range of the Date values before the object is created */
    public Datum(int month, int day, int year) throws IllegalDateException {
        if (!validDate(day, month, year)) 
        	     throw new IllegalDateException("Invalid date", day, month, year);
        	this.month = month;
        	this.day   = day;
        	this.year  = year;
    } 
    
	public int getDay() {
		return day;
	}	
    
    public int getMonth(){
    	return month;
    }

	public int getYear() {
		return year;
	}

    private static boolean validDate(int day, int month, int year) {
         if (month<13){
    	      if (day<1 || day>DAYS[month]) 
        	       return false;
              if (month==2 && day==29 && !isLeapYear(year)) 
        	       return false;
              return true;
         }else return false;
    }

    public static boolean isLeapYear(int year) {
          if (year % 400 == 0) 
        	    return true;
          if (year % 100 == 0) 
        	    return false;
          return (year % 4 == 0);
    }

    /* returns the next Date */
    public void nextDay() {
          if (validDate( day + 1 ,month ,year )) {
        	  day++;
          } else if (validDate(1, month + 1, year)){
        	  day = 1;
        	  month++;
          }else {                                 
        	  day = 1;
        	  month = 1;
        	  year++;
          }
    }
    
    /* set a new Date */
	public void setDate(int day, int month, int year) throws IllegalDateException{
		if (validDate(day, month, year )){
			this.day = day;
			this.month = month;
			this.year = year;
		} else {
			throw new IllegalDateException(day, month, year);
		}		 
	}

    /* check if the invoking object is after date */
    public boolean after(Date date) {
        return compareTo((Datum) date) > 0;
    }

    /* check if the invoking object is before date */
    public boolean before(Date date) {
        return compareTo((Datum) date) < 0;
    }

    /* compare two dates */
    public int compareTo(Datum date) {
        if  (year != date.year)  
        	return year - date.year;
        if  (month != date.month) 
        	return month - date.month;
        return day - date.day;
    }

    /* return a string representation of this date */
    public String toString() {
        return day + "." + month + "." + year;
    }
    
    public Datum clone(){
    	Datum datum = null;
    	try {
			datum = new Datum(day, month, year);
		} catch (IllegalDateException e) {
			e.printStackTrace();
		}
    	return datum;
    }

}// end of class Datum
